
/**
 * Module dependencies.
 */
var daemon = require('daemon')
  , express = require('express')
  , routes = require('./routes')
  , util  = require('util')
  , redis = require("redis")
  , fs = require('fs')
  , exec = require('child_process').exec
  ,_ = require('underscore');

var REDIS_CLIENTS = {};


var createWebServer = function(){

var LOG_FILES = {
        bond_server:{path:'/var/log/sputnik.log', stat:{}}
        ,node:{path:config.outFile, stat:{}}
        ,redis:{path:'/var/log/sputnik-redis.log', stat:{}}
}

var LOG_FILE_LIMIT_SIZE = 536870912; //in bytes

var LOG_FILE_SIZE_CHECKING_INTERVAL = 1 * 24 * 60 * 60 * 1000; //in ms

    var purgeLogs = function(file){
        if( file ){
            var files = {};
            files[file] = LOG_FILES[file];
        }
        else{
            var files = LOG_FILES;
        }
        _.each(files, function(el){
            try{
                var fd = fs.openSync(el.path, 'w');
                //might as well update stats while we're at it
                fs.truncateSync(fd, 0);
                var fstat = fs.fstatSync(fd);
                el.stat = fstat;
                if( fstat.size == 0 ){
                    return true;
                }
                else{
                    return false;
                }
            }
            catch(e){
                console.log('failed to get log file', el, e);
                throw e;
            }
        })
        return LOG_FILES;
    }


    var getLogFilesStats = function(){
        _.each(LOG_FILES, function(el){
            try{
                var fstat = fs.statSync(el.path);
                el.stat = fstat;
            }
            catch(e){
                console.debug('failed to get log file', el, e);
                throw e;
            }
        })
        return LOG_FILES;
    }

    setInterval( function(){
        getLogFilesStats();
        _.each(LOG_FILES, function(el, key){

            if( el.stat.size >=  LOG_FILE_LIMIT_SIZE ){
                console.log('purging ', key, 'because it reached the LOG_LIMIT_SIZE of', LOG_FILE_LIMIT_SIZE);
                purgeLogs(key);
            }
        })

    }, LOG_FILE_SIZE_CHECKING_INTERVAL );





    process.on('uncaughtException', function(err) {
      console.log('Unhandled exception received:' + err);
    });

    var app = module.exports = express.createServer();

    if( !(process.env.NODE_ENV) ){
        app.set('env', 'production');
    }
   //purge the node logs on start up of web server
    purgeLogs('node');


    var io = require('socket.io').listen(app);
      io.configure(function () {
        io.set('transports', ['xhr-polling']);
        });
    var devicesSocket = io.of('/devices');
    app.register('.html', {
        compile: function(str, options){
            try {
                var template = _.template(str);
                return function(locals){
                    return template(locals);
                };
            }
            catch (e) {
                console.log(e);
            }
        }
    });


    // Configuration
    app.set('view options', {
        layout: false
    });
    app.configure(function(){
        app.set('views', __dirname + '/views');
        //app.set('view engine', 'jade');
        app.set('view engine', 'html');
        app.use(express.bodyParser());
        app.use(express.methodOverride());
        app.use(express.bodyParser());
        app.use(express.cookieParser());
        //app.use(express.session());
        app.use('/static', express.static(__dirname + '/static'));
        app.use(express.static(__dirname + '/public'));
        app.use(app.router);
        app.use(express.logger());
        app.use(express.errorHandler());

    });

    app.configure('development', function(){
        app.use(express.errorHandler({
            dumpExceptions: true,
            showStack: true
        }));
        io.set('log level', 10);
    });

    app.configure('production', function(){
        app.use(express.errorHandler());
        io.set('log level', 3);

    });



    app.post('/logs', function(req, res){
        if( config.viewOnly ){
            res.json('{"error":"You are not authorized to perform this action."}');
            return false;
        }

        try{
            if( purgeLogs() ){
                res.send({'state':'success'});
            }
            else{
                res.send({'state':'error'});
            }
        }
        catch(e){
            res.send('error', e);
        }
    });

    app.get('/logs', function(req, res){
        try{
            res.json(getLogFilesStats());
        }
        catch(e){
            res.json(e);
        }
    });

    app.get('/logs/:logID', function(req, res){
        var logID = req.params.logID;
        if(  logID && logID in LOG_FILES){
            res.download(LOG_FILES[req.params.logID].path);
        }
        else{
            res.json({'error': 'log does not exist.'})
        }
    });

    var showLogs = function(req, res){


        /*    r.stream.on( 'connect', function() {
         console.log("node js db connected");
         r.sort( 'id:1:log', function( err, data ) {
         console.log("nodejs sorting");
         if( !data ) {
         res.writeHead( 404 );
         res.write( "No such snippet" );
         res.end();
         r.quit();
         return;
         }

         console.log("nodejs data", data);
         res.writeHead( 200 );
         res.write( 'data' );
         res.write(util.inspect(data));
         res.end();
         r.quit();
         });
     });*/


    }



    var dbc = redis.createClient(config.redisPort);
    dbc.on("error", function(err){
         console.log( 'database error:', err);
         return false;
    });


    var ACTIVE_SESSIONS = {};
    var get_active_sessions = function(callback){
        dbc.get('sess:active',
            function(err, sessionIDs){
                if( err || !sessionIDs ){
                    console.log( 'error:', err);
                    return false;
                }
                ACTIVE_SESSIONS = {};
                var activeSessions = sessionIDs.split(',');
                _.each(activeSessions, function(sess){
                    matches = sess.match(/sess:([0-9]+):([A-Z]+):([0-9]+).*/);
                    if( matches.length == 4 ){
                        var deviceID = matches[1];
                        var capabilities = matches[2];
                        if( !(ACTIVE_SESSIONS[deviceID]) ){ACTIVE_SESSIONS[deviceID]={}}
                        var sessData = {
                            device: matches[1]
                            ,capabilities: matches[2]
                            ,session_id: matches[3]
                            ,db_key: 'sess:'+matches[1]+':'+matches[2]
                        }
                        if( capabilities.search(/C/) != -1 ){
                            ACTIVE_SESSIONS[deviceID].command = sessData;
                        }
                        if( capabilities.search(/E/) != -1 ){
                            ACTIVE_SESSIONS[deviceID].video = sessData;
                        }
                        if( capabilities.search(/I/) != -1 ){
                            ACTIVE_SESSIONS[deviceID].ifb = sessData;
                        }

                    }
                });
                if( typeof(callback) == "function" ){
                    callback(ACTIVE_SESSIONS);
                }
        });

    };
    get_active_sessions();

    // Routes

    app.get('/', function(req, res){
        res.render('dashboard', {});
    });
    app.get('/version', function(req, res){
        //res.set("Content-Type", "text/plain");
        res.header("Content-Type", "text/plain");
        res.sendfile('/opt/teradek/sputnik/VERSION');
    });
    app.get('/session', function(req, res){
        res.render('session', {});
    });

    app.get('/session/:sessID', function(req, res){
        res.render('session', {
            sessID: req.params.sessID
        });
    });

    app.get('/session/:sessID/:connID', function(req, res){
        res.render('connection', {
            sessID: req.params.sessID
            ,connID: req.params.connID
            ,startTime: req.params.startTime
            ,endTime:   req.params.endTime

        });
    });

    app.get('/config/viewOnly', function(req,res){
       res.writeHead(200, {"Content-Type": "text/plain"});
        res.write(config.viewOnly.toString());
        res.end();
    })

    app.get('/restart', function(req, res){
        if( config.viewOnly ){
            res.writeHead(200, {"Content-Type": "application/json"});
            res.write('{"error":"You are not authorized to perform this action."}');
            res.end();
            return false;
        }
        exec("service sputnik restart", function(error, stdout, stderr){
			    res.writeHead(200, {"Content-Type": "application/json"});
				if (error) {
					res.write('{"error":"' + error + '"}');
				}
				else{
					var output = stdout.replace(/\n/g, " ");
					res.write('{"success":"' + output + '"}');
				}
                res.end();
		});
    });


    app.get('/connection', function(req, res){
        res.render('connection', {});
    });
    app.get('/dashboard', function(req, res){
        res.render('dashboard', {});
    });


    app.get('/json/:sessID/port/:portID', function(req, res){
        var sessID = req.params.sessID;
        var portID = req.params.portID;
        if( sessionArgsIsInvalid(sessID) ){
            res.write('{"error":"session is invalid."}');
            res.end();
            return false;
        }
        var db_key = ACTIVE_SESSIONS[sessID].video.db_key;

        res.writeHead(200, {"Content-Type": "application/json"});
        try{
            console.log( sessID, portID, 'sessid', 'portid')

            dbc.zrange(db_key+ ':conn:'+portID+':modem_info', 0, 0,
                     function(err, reply){
                        if( err ){ console.log( 'error:', err); return false;}
                        if (reply && reply[0] && typeof(reply[0]) == 'string') {
                            res.write(reply[0]);
                        }
                        res.end();
                      });
        }
        catch(e){
            res.write('{"error":"unable to create database client"}');res.end();
            return false;
        }
    });

    app.get('/json/sessions/totals', function(req, res){
        try{
            var resObj = {};
            get_active_sessions(function(sessionKeys){

                if( sessionKeys == null || Object.keys(sessionKeys).length === 0){
                    res.json({});
                    return false;
                }
                var resObj = {};
                var completeResponse = function(){
                    //make sure all sessions have an array (not null) signifying all db queries done
                    if( _.every(resObj, function(sess){return sess != null;}) ){
                        res.json(resObj);
                    }
                }
                _.each(sessionKeys, function(sess, sessionID){
                    resObj[sessionID] = null;
                });
                _.each(sessionKeys, function(sess, sessionID){
                    var video_session = sess.video;
                    if( video_session ){
                        dbc.zrange(video_session.db_key+':conn:total:stats', -20, -1, function(err, replies){
                            if( replies ){
                                //convert response with array of JSON string into valid JSON
                                var data = [];
                                _.each(replies, function(r){
                                    var jsonData = JSON.parse(r);
                                    if( jsonData.session_id == video_session.session_id ){
                                        data.push(jsonData);
                                    }
                                });
                                resObj[sessionID] = data;
                            }
                            else{
                                resObj[sessionID] = [];
                            }
                            completeResponse();
                        });
                    }
                    else{
                        resObj[sessionID] = {error:"Video data unavailable."};
                        completeResponse();
                    }

                })
            });
        }
        catch(e){
            console.log(e);
            res.write('{"error":"unable to create database client"}');res.end();
            return false;
        }
    });

    app.get('/uptime/:sessID', function(req, res){
        res.writeHead(200, {"Content-Type": "application/json"});
        var sessionID = req.params.sessID;
        if( sessionArgsIsInvalid(sessID) ){
            res.write('{"error":"session is invalid."}');
            res.end();
            return false;
        }
        var db_key = ACTIVE_SESSIONS[sessID].video.db_key;
        dbc.zrange(db_key + ':conn:total:stats', 0, 0, function(err, reply){
            if (reply && reply[0] && typeof(reply[0]) == 'string') {
                res.write(reply[0]);
            }
            res.end();
        });


    });

    app.get('/time', function(req, res){
        res.writeHead(200, {"Content-Type": "text/plain"});
        var ts = new Date().getTime();
        res.write(ts.toString());
        res.end();
    });


    var sessionArgsIsInvalid = function(sessID){
        return ( !(sessID) ||
                !( ACTIVE_SESSIONS[sessID] ) ||
                !( ACTIVE_SESSIONS[sessID].video) ||
                !( ACTIVE_SESSIONS[sessID].video.db_key));
    }


    /*
     *  note: important to distinguish that 'connection' is not the same as 'port(ppp#)'
     *  as theoretically you may have multiple connections per port
     *  but as a shortcut connection is ignored
     */
    app.get('/json/:sessID', function(req, res){
        res.writeHead(200, {"Content-Type": "application/json"});
        var startTime = req.query['startTime'];
        var endTime = req.query['endTime'];
        if( !(endTime) ){
          endTime = "+inf";
        }
        var sessID = req.params.sessID;
        if( sessionArgsIsInvalid(sessID) ){
            res.write('{"error":"session is invalid."}');
            res.end();
            return false;
        }
        var db_key = ACTIVE_SESSIONS[sessID].video.db_key;
        dbc.keys(db_key+':conn:*', function(err, conns){
            if (!(conns) || !(conns.length)) {
                console.log('no connections found', err);
                if ( err ){
                    res.write('{"error":"'+err+'"}');
                }
                else{
                    res.write("{}");
                }
                res.end();
                return false;
            }
            var response = "{";
            connsLength = conns.length;
            if( err ){ console.log( 'error:', err); return false;}
            conns.forEach(function(connKey, connIndex){
                dbc.zrangebyscore(connKey, startTime, endTime, function(err, replies){
                    if(connKey.search('total') != -1){
                        var portName = 'total';
                    }
                    else{
                        var portName = "Unknown";
                       replies.every(function(replyStr){
                         try{
                         var firstReply = JSON.parse(replyStr);

                         if( firstReply.name && firstReply.name != "" ){
                              portName = firstReply.name;
                              return false;
                         }
                         return true;
                         }
                         catch(e){
                             console.log( e );

                         }
                       })
                    }
                    response += '"' + portName + '":';
                    response += '[ ';
                    replies.forEach(function(reply, i){
                        response += reply + ',';
                    });
                    response = response.substring(0, response.length - 1);
                    response += '],';
                    //last connection
                    if (connIndex == connsLength - 1) {
                        response = response.substring(0, response.length - 1);
                        response += '}';
                        res.write(response);
                        res.end();
                    }
                });
            });
        });
    });


    app.get('/client', function(req, res){
        res.render('client', {});
    });







    var socketArgsIsInvalid = function(data){
        return (typeof data != 'object' ||
                !(data.sessID) ||
                !( ACTIVE_SESSIONS[data.sessID] ) ||
                !( ACTIVE_SESSIONS[data.sessID].command) ||
                !( ACTIVE_SESSIONS[data.sessID].command.db_key));
    }

    var getRedisClient = function(sessID){
        if( ACTIVE_SESSIONS[sessID] ){
            var db_subscribe = ACTIVE_SESSIONS[sessID].command.db_key;
            if( !(REDIS_CLIENTS[ db_subscribe ]) ){
                REDIS_CLIENTS[ db_subscribe ] = {'pub':redis.createClient(), 'sub':redis.createClient()};
                REDIS_CLIENTS[ db_subscribe ]['sub'].subscribe(db_subscribe);
                REDIS_CLIENTS[ db_subscribe ]['sub'].on("message", function( channel, message ){

                    if( message.substr(0,5) == 'reply' ){
                        var startPayload = message.indexOf(':', 6)+1;
                        var payload = JSON.parse(message.substr(startPayload));
                        //if Link Mapping request
                        if( payload.cube_id ){
                           payload.link_id = channel.split(':')[1];
                           devicesSocket.emit('new_settings', payload);
                        }
                        else{
                            devicesSocket.to(sessID).emit('new_settings', payload);
                        }
                    }
                    else if( message == "close_session" ){
                        devicesSocket.to(sessID).emit('close_session');
                    }
                    else if( message == "open_session" ){
                        devicesSocket.to(sessID).emit('open_session');
                    }
                })
            }
            return REDIS_CLIENTS[ db_subscribe ];
        }
        return redis.createClient();
    }


    devicesSocket.on('connection', function(socket){
        socket.on('leave', function(data){
            socket.leave(data.sessID);
        })

        socket.on('join', function(data, fn){
            if( socketArgsIsInvalid(data) ){
                fn({error:'Session does not exist or does not have remote control enabled.'});
                return false;
            }
             var sessID = data.sessID;
            socket.join(sessID);
            socket.broadcast.to(sessID).emit('joined', socket.id);
            getRedisClient(sessID);
            if( fn ){
                fn(socket.id);
            }

        })


         /*  ---------- Clients will request these ------------- */
        socket.on('client_get_settings', function(data){
            if( socketArgsIsInvalid(data) ){
                return false;
            }
            var sessID = data.sessID;
            var jsonCmd = data.jsonCmd;
            var db_publish = ACTIVE_SESSIONS[data.sessID].command.db_key;
            var rc = getRedisClient(sessID);
            rc['pub'].publish(db_publish,'req:124:2:1:3:'+jsonCmd);
        })

        socket.on('client_change_settings', function(data){
            if( config.viewOnly ){
                socket.send("unauthorized");
                return false;
            }
            if( socketArgsIsInvalid(data) ){
                return false;
            }
            var db_publish = ACTIVE_SESSIONS[data.sessID].command.db_key;
            var sessID = data.sessID;
            var jsonCmd = data.jsonCmd;
            var rc = getRedisClient(sessID);
            socket.broadcast.to(sessID).emit('changing_settings');
            rc['pub'].publish(db_publish,'req:124:2:1:3:'+jsonCmd);
        });

        socket.on('client_action', function(data){
            if( config.viewOnly ){
                socket.send("unauthorized");
                return false;
            }
            if( socketArgsIsInvalid(data) ){
                return false;
            }
            var sessID = data.sessID;
            var jsonCmd = data.jsonCmd;
            var ioctl = ( data.ioctl ) ? data.ioctl : 4;
            var db_publish = ACTIVE_SESSIONS[data.sessID].command.db_key;
            var rc = getRedisClient(sessID);
            rc['pub'].publish(db_publish,'req:124:2:'+ioctl+':3:'+jsonCmd);
        })

        socket.on('client_get_link_cube', function(data){
            var linkID = data.linkID;
            var jsonCmd = data.jsonCmd;
            var db_publish = ACTIVE_SESSIONS[linkID].command.db_key;
            var rc = getRedisClient( linkID );
            rc['pub'].publish(db_publish,'req:123:2:5:3:'+jsonCmd);
        })

    });

     app.listen(config.port);
    console.log("Express server listening on port %d in %s mode", config.port, app.settings.env);

}

var args = process.argv;
var argsObj = {};
var params = [];




for( var i in args ){
  if( args[i].indexOf( "-" ) == 0 ){
        var arg = args[i].substring( 1 ).split( "=" );
        argsObj[arg[0]] = arg[1];
   }
   else{
    params.push(args[i]);
   }
}

var config = {
     port:     argsObj['port']    || 1957
    ,redisPort:argsObj['redis']   || 6379
    ,lockFile: argsObj['pid']     || '/tmp/sputnik-node.pid'
    ,outFile:  argsObj['log']     || '/tmp/sputnik-node-output.log'
    ,errFile:  argsObj['err']     || null
    ,viewOnly: false
};
if( argsObj.hasOwnProperty('view-only') ){
    config.viewOnly = true;
}

switch(params[2]) {
    case "stop":
        daemon.kill( config.lockFile, function( err, pid ){
          if (err) {
            return util.puts('Error stopping daemon: ' + err);
          }

          util.puts('Successfully stopped daemon with pid: ' + pid);
        });
        break;
    case "start":
        var fds = config.errFile ? { stdout: config.outFile, stderr: config.errFile } : config.outFile;
        //purge sputnik node logs when node started
        daemon.daemonize( fds, config.lockFile, function(err, pid ){
            if (err) {
              return console.log('Error starting daemon: ' + err);
            }
            console.log('Sputnik Node.js Daemon started successfully with pid: ' + pid);
         });
         createWebServer();
        break;
    case "starti":
        createWebServer();
        break;
    default:
        util.puts('Usage: [start|starti|stop] [ flags ]');
        util.puts(' -port=number');
        util.puts('        Set the network port number to listen on');
        util.puts(' -redis=number');
        util.puts('        Set the redis port number to connect to');
        util.puts(' -pid=file');
        util.puts('        Set the location of the pid file');
        util.puts(' -log=file');
        util.puts('        Set the location of the stdout "log" file');
        util.puts(' -err=file');
        util.puts('        Set the location of the separate stderr file');
		util.puts(' -view-only');
		util.puts('        Prevent changing settings from the dashboard.');

        process.exit(0);
}
